﻿using System;
using System.ComponentModel.Composition;
using System.ComponentModel.Composition.Hosting;
using System.Diagnostics;
using System.Web.Mvc;
using Adressverwaltung.Data.Models;
using Adressverwaltung.Data.Repositories;

namespace Adressverwaltung.Web.Controllers
{
    public class AdressbuchController : Controller
    {
        [Import(typeof(IAdressverwaltungRepository))]
        private IAdressverwaltungRepository _repository;

        [Import(typeof(IFotoRepository))]
        private IFotoRepository _fotos;

        public AdressbuchController()
        {
            using (var catalog = new DirectoryCatalog("bin", "Adressverwaltung.Data.dll"))
            using (var container = new CompositionContainer(catalog))
                container.ComposeParts(this);
        }

        public ActionResult Fehler()
        {
            return View();
        }

        public ActionResult Index()
        {
            try
            {
                return View(_repository.GetAdressbuecher());
            }
            catch (Exception ex)
            {
                Trace.TraceError(ex.Message);
                return RedirectToAction("Fehler");
            }
        }

        public ActionResult Details(Guid adressbuchId)
        {
            try
            {
                return View(_repository.LoadAdressbuch(adressbuchId));
            }
            catch (Exception ex)
            {
                Trace.TraceError(ex.Message);
                return RedirectToAction("Fehler");
            }
        }

        public ActionResult Erstellen()
        {
            return View();
        }

        [HttpPost]
        public ActionResult Erstellen(Adressbuch adressbuch)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    _repository.AddAdressbuch(adressbuch);

                    return RedirectToAction("Index");
                }
                return View(adressbuch);
            }
            catch (Exception ex)
            {
                Trace.TraceError(ex.Message);
                return RedirectToAction("Fehler");
            }
        }

        public ActionResult Aendern(Guid adressbuchId)
        {
            try
            {
                return View(_repository.LoadAdressbuch(adressbuchId));
            }
            catch (Exception ex)
            {
                Trace.TraceError(ex.Message);
                return RedirectToAction("Fehler");
            }
        }

        [HttpPost]
        public ActionResult Aendern(Guid adressbuchId, Adressbuch adressbuch)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    _repository.UpdateAdressbuch(adressbuchId, adressbuch);

                    return RedirectToAction("Details", new { adressbuchId });
                }

                return View(adressbuch);
            }
            catch (Exception ex)
            {
                Trace.TraceError(ex.Message);
                return RedirectToAction("Fehler");
            }
        }

        public ActionResult Loeschen(Guid adressbuchId)
        {
            try
            {
                return View(_repository.LoadAdressbuch(adressbuchId));
            }
            catch (Exception ex)
            {
                Trace.TraceError(ex.Message);
                return RedirectToAction("Fehler");
            }
        }

        [HttpPost]
        public ActionResult Loeschen(Guid adressbuchId, Adressbuch adressbuch)
        {
            try
            {
                _fotos.AdressbuchBilderEntfernen(adressbuchId);
                _repository.DeleteAdressbuch(adressbuchId);

                return RedirectToAction("Index");
            }
            catch (Exception ex)
            {
                Trace.TraceError(ex.Message);
                return RedirectToAction("Fehler");
            }
        }
    }
}
