﻿using System;
using System.ComponentModel.Composition;
using System.ComponentModel.Composition.Hosting;
using System.Diagnostics;
using System.Web;
using System.Web.Mvc;
using Adressverwaltung.Data.Models;
using Adressverwaltung.Data.Repositories;

namespace Adressverwaltung.Web.Controllers
{
    public class KontaktController : Controller
    {
        [Import(typeof(IAdressverwaltungRepository))]
        private IAdressverwaltungRepository _repository;

        [Import(typeof(IFotoRepository))]
        private IFotoRepository _fotos;

        public KontaktController()
        {
            using (var catalog = new DirectoryCatalog("bin", "Adressverwaltung.Data.dll"))
            using (var container = new CompositionContainer(catalog))
                container.ComposeParts(this);
        }

        public ActionResult Fehler()
        {
            return View();
        }

        public ActionResult Index(Guid adressbuchId)
        {
            try
            {
                TempData["AdressbuchId"] = adressbuchId.ToString();
                return View(_repository.GetKontakte(adressbuchId));
            }
            catch (Exception ex)
            {
                Trace.TraceError(ex.Message);
                return RedirectToAction("Fehler");
            }
        }

        public ActionResult Details(Guid adressbuchId, Guid kontaktId)
        {
            try
            {
                return View(_repository.LoadKontakt(adressbuchId, kontaktId));
            }
            catch (Exception ex)
            {
                Trace.TraceError(ex.Message);
                return RedirectToAction("Fehler");
            }
        }

        public ActionResult Erstellen(Guid adressbuchId)
        {
            TempData["AdressbuchId"] = adressbuchId.ToString();
            return View();
        }

        [HttpPost]
        public ActionResult Erstellen(Guid adressbuchId, Kontakt kontakt, HttpPostedFileBase foto)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    _repository.AddKontakt(adressbuchId, kontakt);

                    if (foto != null)
                    {
                        _fotos.KontaktBildHinzufuegen(
                            foto.InputStream,
                            foto.FileName,
                            foto.ContentType,
                            kontakt);
                    }

                    return RedirectToAction("Index", new { adressbuchId });
                }

                TempData["AdressbuchId"] = adressbuchId.ToString();
                return View(kontakt);
            }
            catch (Exception ex)
            {
                Trace.TraceError(ex.Message);
                return RedirectToAction("Fehler");
            }
        }

        public ActionResult Aendern(Guid adressbuchId, Guid kontaktId)
        {
            try
            {
                return View(_repository.LoadKontakt(adressbuchId, kontaktId));
            }
            catch (Exception ex)
            {
                Trace.TraceError(ex.Message);
                return RedirectToAction("Fehler");
            }
        }

        [HttpPost]
        public ActionResult Aendern(Guid adressbuchId, Guid kontaktId, Kontakt kontakt, HttpPostedFileBase foto)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    var repository = _repository;
                    repository.UpdateKontakt(adressbuchId, kontaktId, kontakt);

                    if (foto != null)
                    {
                        _fotos.KontaktBildHinzufuegen(
                            foto.InputStream,
                            foto.FileName,
                            foto.ContentType,
                            repository.LoadKontakt(adressbuchId, kontaktId));
                    }

                    return RedirectToAction("Details", new { adressbuchId, kontaktId });
                }

                return View(kontakt);
            }
            catch (Exception ex)
            {
                Trace.TraceError(ex.Message);
                return RedirectToAction("Fehler");
            }
        }

        public ActionResult Loeschen(Guid adressbuchId, Guid kontaktId)
        {
            try
            {
                return View(_repository.LoadKontakt(adressbuchId, kontaktId));
            }
            catch (Exception ex)
            {
                Trace.TraceError(ex.Message);
                return RedirectToAction("Fehler");
            }
        }

        [HttpPost]
        public ActionResult Loeschen(Guid adressbuchId, Guid kontaktId, Kontakt kontakt)
        {
            try
            {
                _fotos.KontaktBildEntfernen(kontaktId);
                _repository.DeleteKontakt(adressbuchId, kontaktId);

                return RedirectToAction("Index", new { adressbuchId });
            }
            catch (Exception ex)
            {
                Trace.TraceError(ex.Message);
                return RedirectToAction("Fehler");
            }
        }
    }
}
